#pragma once
#include <iostream>
#include <memory>
#include <string>
#include <array>
#include <vector>
#include "../drawable/menu.h"
#include "../states/istatemanager.h"
#include "../states/state.h"
#include <SFML/Graphics.hpp>


class Application : public IStateManager {

public:
	int run();
private:
	void set_next_state(std::unique_ptr<IState> ptr_state) {m_ptr_state_next = std::move(ptr_state);}
	void apply_deffer_state_change() { m_ptr_state_current = std::move(m_ptr_state_next);  }
private:
	std::unique_ptr<IState> m_ptr_state_next = nullptr;
	std::unique_ptr<IState> m_ptr_state_current = nullptr;
};


class MyFont {
public: //singletone - we get the unique instance of the class
	sf::Font& Instance() {
		static sf::Font font;
		return font;
	}
	MyFont(const MyFont& font) = delete;
	MyFont& operator=(const MyFont&) = delete;
private:
	MyFont() { m_font.loadFromFile("LibreBaskerville-Regular.ttf"); }
private:
	sf::Font m_font;
};

class Room;

class IEntity: public IPreparable {
public:
	void set_location(std::unique_ptr<Room> ptr_room){}
	Room* get_location() {
	}
protected: 
	Room* m_ptr_room;
};

class IRoomSide : public IPreparable {
public:
	virtual void enter(IEntity* entity) = 0;
	virtual ~IRoomSide() = 0 {}

};

class Room :public IDrawable {
public:
	enum Direction { INVALID = -1, LEFT, RIGHT, UP, DOWN };

	Room(float size){}
	float get_size() {}
	void set_position(sf::Vector2f pos) {}
	sf::Vector2f get_position() {}
	void set_side(Direction side, std::unique_ptr<IRoomSide> ptr_side) {}
	IRoomSide* get_side(Direction side) { return m_sides[0]; }
	Direction get_direction(std::unique_ptr<IRoomSide> ptr_side){}

	void draw_into(sf::RenderWindow& window) const override;

	std::array<IRoomSide*, 4> m_sides;

private:
	sf::RectangleShape m_rectangle;

};

class Maze : public IDrawable {
public:
	Maze(std::vector<std::unique_ptr<Room>> rooms){}
	void draw_into(sf::RenderWindow& window) const override;

private:
	std::vector<Room> m_rooms;
};

class Pass : public IRoomSide {
public:
	Pass(Room& room1, Room& room2) : m_room1(room1), m_room2(room2) {}
	virtual void prepare_for_drawing() override {}
	virtual void draw_into(sf::RenderWindow& window) const override {}
private:
	Room& m_room1;
	Room& m_room2;
};

class Wall : public IRoomSide {


};

class Pacman : public IEntity {
	Pacman() {}
	virtual void prepare_for_drawing() override {}
	virtual void draw_into(sf::RenderWindow& window) const override {}
	void move(Room::Direction direction);
};

class IStaticEntity : public IEntity {
public:
	virtual std::unique_ptr<IStaticEntity> clone() const = 0;
	virtual ~IStaticEntity() = 0 {}
};

class IDynamicEntity : public IEntity {
public:
	virtual std::unique_ptr<IDynamicEntity> clone() const = 0;
	void virtual action() = 0;
	virtual ~IDynamicEntity() = 0 {}
};

class Food : public IStaticEntity {

};
//
//class Enemy: public IDynamicEntity {
//	Enemy() {}
//	void action() override;
//	virtual std::unique_ptr<IDynamicEntity> clone() const override {return std::make_unique<Enemy>(*this); }
//};

struct GameContext {
	Pacman pacman;
	std::vector<std::unique_ptr<IStaticEntity>> static_objects;
	std::vector<std::unique_ptr<IDynamicEntity>> dynamic_objects;
	enum State { INGAME, WIN, LOST } state = INGAME;

	GameContext clone();
};